-- show_xml_table
--     procedure definition and service
--
-- also requires qxml service to be defined if that functionality is used
--
call sa_make_object( 'procedure', 'show_xml_table' )
go

alter procedure show_xml_table(user_name long varchar, table_name long varchar, limit int, start_at int, showxml long varchar)
result (xml_document long varchar)
BEGIN


    --these are parameters for the xml
    declare xml_header long varchar;
    declare xml_translate long varchar;
    declare xml_params long varchar;
    declare xml_result long varchar;

    --parameters used in the sql query
    declare sql_query long varchar;
    declare sql_results long varchar;
    declare sql_top_n long varchar;
    declare sql_user long varchar;
    declare sql_start_at long varchar;


    --for the  browser
    call dbo.sa_set_http_header( 'Content-Type', 'text/xml' );

    --for null handling
    set option for_xml_null_treatment='empty';

    --create the xml header since our webservice returns data in raw format.  FOR XML does not return proper
    --xml in the sense that there is no one root node as is required by xml standard
    set xml_header = '<?xml version=''1.0'' encoding=''windows-1252'' standalone=''yes''?>\n';

    --apply an xsl translation to the xml output
    set xml_translate = ifnull(showxml, '<?xml-stylesheet type="text/xsl" href="xsldemo"?>\n', ''); 
    
    --store tablename and username etc in xml format for the xsl to access
    set xml_params = '<table_properties>'||
                        '<user_name>'||user_name||'</user_name>'|| 
                        '<table_name>'||table_name||'</table_name>
                        <limit>'||limit||'</limit>
                        <start_at>'||start_at||'</start_at>'||
                     '</table_properties>';

    
    set sql_top_n = ifnull(limit, '', ' top ' || cast( limit as int ));

    set sql_start_at = ifnull (start_at, '', ' start at ' || cast (start_at as int) || ' ');


    set sql_user = ifnull (user_name, '', 'and t.creator = ( select user_id'|| 
                                        ' from sysuserperm where user_name = ''' || user_name || ''' )' );

    --first query:  select all the tables for a given username (if not all)
    set sql_query = ifnull( table_name,  '(select u.user_name, t.table_name '||
                                     ' FROM sys.sysuserperm u, sys.systable t '||
                                     ' WHERE  u.user_id = t.creator '||
                                       sql_user ||
                                     ' ORDER BY u.user_name, t.table_name '||
                                     ' FOR XML AUTO) ',

    --second query:  selects the column names (c) and contents for one table
    --this may be null if there is nothing in the tables
                                     ' (select c.column_name '||
                                     ' FROM sys.syscolumn c, sys.systable t '||
                                     ' WHERE t.table_id = c.table_id '||
                                       sql_user || 
                                     ' AND t.table_name = ''' ||table_name|| ''''||
                                     ' FOR XML AUTO) || ' ||
                                     ' (SELECT ' || sql_top_n || sql_start_at || ' * '||
                                     ' FROM ' ||table_name ||' FOR XML AUTO)'); 


    --select sql_query;            --debugging


    execute immediate 'set sql_results =  ' || sql_query;

    
    --result formed using string concatenation
    set xml_result =  xml_header || --xml declaration
                xml_translate|| --xsl translation
                '<Tables>'||    --root node
                xml_params ||   --stores parameters in xml format
                sql_results ||  --results of the xml query
                '</Tables>';    --close root node

    select xml_result;

    RETURN;

END;

go

--will throw an error if it's already been created and you're creating a new one
call sa_make_object( 'service', 'show_xml_table' )
go
alter service show_xml_table type 'raw' authorization on user dba secure off
  as call show_xml_table(:user_name, :table_name, :limit, :start_at, :showxml)
go

--
-- qxml
--     procedure definition and service
--

--alters the actual service.  just need to specify the
--service name and return type as xml
call sa_make_object( 'service', 'qxml' )
go
alter service qxml type 'xml' authorization on user dba secure off
go

-- xsldemo
--     procedure definition and service
--
-- also requires qxml service to be defined if that functionality is used
-- 
call sa_make_object( 'procedure', 'xsldemo' )
go

alter procedure xsldemo( )
result (xml_document long varchar)
BEGIN

    
   --set the http header for xml, so that it returns an xml (xsl) doc
   call dbo.sa_set_http_header( 'Content-Type', 'text/xml' );
            
    SELECT '<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
            <xsl:template match="/">
                <html>
                    <Title>Database Tables using XML</Title>
                    <body>' ||

                      --table_properties info, stored in variables
                        ' <xsl:variable name="user_name" select="Tables/table_properties/user_name" />'||    
                        ' <xsl:variable name="table_name" select="Tables/table_properties/table_name" />'||
                        ' <xsl:variable name="limit_n" select="number(Tables/table_properties/limit)" />'||
                        ' <xsl:variable name="start_at" select="number(Tables/table_properties/start_at)" />'||
                        
                        --query strings for links & parameters
                        ' <xsl:variable name="qtable">'||           
                             ' <xsl:if test="$table_name != ''''">'||
                             '  <![CDATA[&table_name=]]><xsl:value-of select="$table_name"/>'||
                             ' </xsl:if>'||
                        ' </xsl:variable>' || 

                        ' <xsl:variable name="quser">'||
                             ' <xsl:if test="$user_name != ''''">'||
                             '    <![CDATA[user_name=]]><xsl:value-of select="$user_name"/>'||
                             ' </xsl:if>'||
                        ' </xsl:variable>' ||

                        ' <xsl:variable name="qlimit">'||
                             ' <xsl:if test="$limit_n != ''''">'||
                             '    <![CDATA[&limit=]]><xsl:value-of select="$limit_n"/>'||
                             ' </xsl:if>'||
                        ' </xsl:variable>'||

                      --this creates the yellow table along the top and/or bottom
                      --in a variable $yellowtable, instead of copying this code over and over
                        ' <xsl:variable name="yellowtable">'||
                        ' <table width="100%" bgcolor="#FFFF11">
                            <tr>'||
                              --create link:  "FULL LIST OF TABLES"
                              ' <td align="center"> <a href="show_xml_table">Full List of Tables </a> </td>'||
                             
                              --create link:  "LIST OF TABLES FOR USER ____ " 
                              ' <xsl:if test="$user_name != ''''">'||
                              ' <td align="center"> 
                                 <a> 
                                    <xsl:attribute name="href">  
                                        <![CDATA[show_xml_table?]]><xsl:copy-of select="$quser"/>
                                    </xsl:attribute>
                                    List Tables For User <xsl:value-of select="$user_name" />
                                 </a>
                                </td>'||
                              ' </xsl:if>'||
                              
                              ' <xsl:if test="Tables/table_properties/start_at  != '''' or Tables/table_properties/limit != ''''">'||
                                --for showing n records at a time, figure out the jump before and after
                                --with a bunch of choose/when statements
                                    ' <xsl:variable name="qstartbefore">'||
                                        ' <xsl:choose>'||
                                            ' <xsl:when test="$start_at &gt; $limit_n">'||
                                                '<xsl:value-of select="$start_at - $limit_n"/>'||
                                            ' </xsl:when>'|| 
                                            ' <xsl:otherwise>'||
                                                '1'||
                                            ' </xsl:otherwise>'||
                                        ' </xsl:choose>'||
                                    ' </xsl:variable>'||
                                    ' <xsl:variable name="qstartafter" select="$start_at + $limit_n"/>'|| 


                                 --create link:  "PREVIOUS ROWS"
                                   '<xsl:if test="$start_at &gt; 1">'||
                                   '<td align="center"><a>
                                        <xsl:attribute name="href">
                                            <![CDATA[show_xml_table?]]><xsl:copy-of select="$quser"/><xsl:copy-of select="$qtable"/><xsl:copy-of select="$qlimit"/><![CDATA[&start_at=]]><xsl:copy-of select="$qstartbefore"/>
                                        </xsl:attribute>
                                      Previous Rows'||
                                    '</a></td>'||
                                    '</xsl:if>'||



                                 --create link:  "NEXT ROWS"
                                    '<td align="center"><a>
                                        <xsl:attribute name="href">
                                            <![CDATA[show_xml_table?]]><xsl:copy-of select="$quser"/><xsl:copy-of select="$qtable"/><xsl:copy-of select="$qlimit"/><![CDATA[&start_at=]]><xsl:copy-of select="$qstartafter"/>
                                        </xsl:attribute>
                                      Next Rows'||
                                    '</a></td>'||
                              '</xsl:if>'||


                                --create link:  "SERVICES"
                                '<td align="center"><a>'||
                                    '<xsl:attribute name="href">'|| 
                                    --build the links using cdata and xsl:value-of, since xml doesn't like the & and = in links   
                                    '<![CDATA[show_xml_table?user_name=SYS&table_name=SysWebService]]>
                                    </xsl:attribute>
                                  Services
                                </a></td>'||

                                --create link:  "Show Users"
                                '<td align="center">
                                    <a>
                                    <xsl:attribute name="href">
                                    <![CDATA[show_xml_table?user_name=DBO&table_name=SysUsers]]>
                                    </xsl:attribute>
                                    Show Users
                                </a>
                                </td>'||

                                --put more links here 

                            '</tr>
                        </table>
                        </xsl:variable>'||
                        --/$yellowtable
                        
                    --insert the html code for $yellowtable
                    '<xsl:copy-of select="$yellowtable"/>'||
                    
                    --2 cases for show_xml_table:  show the contents of one table or a subset of its records,
                    --                             or show all the tables or a subset of the tables (sort by user)
                    '<xsl:choose>' ||

                        --table name is specified, so show the table's contents
                        '<xsl:when test="$table_name != ''''">'||   
                            '<h1>Table "<xsl:value-of select="$user_name"/>".<xsl:value-of select="$table_name"/></h1>'|| 
                            ' <a>
                                  <xsl:attribute name="href">
                                        <![CDATA[show_xml_table?]]><xsl:value-of select="$quser"/><xsl:value-of select="$qtable"/><![CDATA[&limit=]]><xsl:value-of select="Tables/table_properties/limit"/><![CDATA[&start_at=]]><xsl:value-of select="Tables/table_properties/start_at"/><![CDATA[&showxml=TRUE]]>'||                                  
                                ' </xsl:attribute>'||
                                '(view the xml)'||
                            '</a><br/><br/>'||
                        
                            '<table border="1">'||
        
                            --create the table column headings
                            --iterate through all the c elements (created with the SELECT c.column_name statement)
                            '<xsl:for-each select="Tables/c">'||
                                '<td><b><xsl:value-of select="@column_name"/></b></td>'||
                            '</xsl:for-each>'||
            
                            --create the table's contents  
                            --we want to iterate through all nodes whose name() is the that of $table_name
                            --ie using  Tables/*[condition] construct
                            '<xsl:for-each select="Tables/*[name()=$table_name]">'||
                            
                            --now we go through each attribute for these elements, ie @*    
                            --to construct the actual table
                                '<tr><xsl:for-each select="@*">'||                      
                                    '<td>'||
                                        '<xsl:value-of select="."/>'||
                                        '<xsl:if test=". = ''''">'||
                                            '<i>-null-</i>'||
                                        '</xsl:if>'||
                                    '<br/></td>'||
                                '</xsl:for-each></tr>'||
                            '</xsl:for-each>'|| 
                            '</table>'||
                        '</xsl:when>'||

                        
                        --xsl:otherwise - show all the tables or a subset of these
                        '<xsl:otherwise>'||
                          ' <h1>Database Tables using XML</h1>'||
                            ' <a>
                                  <xsl:attribute name="href">
                                        <![CDATA[show_xml_table?showxml=TRUE]]>'||                                  
                                ' </xsl:attribute>'||
                                '(view the xml)'||
                            '</a><br/><br/>'||

                            '<u>Link Legend:</u><br/>'||

                            --create the legend, this is just lots of text and formatting it so that it works
                            --both in html and with xml to get the lil' symbols
                            --we use CDATA to handle the html special characters for xml processing and 
                            --xsl:text so characters are handled properly by the browser
                          ' <xsl:text disable-output-escaping="yes"> 
                                <![CDATA[&oplus;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;]]>
                            </xsl:text>
                            Query table using "qxml" XML service.<br/>

                            <xsl:text disable-output-escaping="yes"> 
                                <![CDATA[&Delta;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;]]>
                            </xsl:text>
                            Show first 20 rows of table using show_table service.<br/>

                            <xsl:text disable-output-escaping="yes"> 
                                <![CDATA[&Dagger;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;]]>
                            </xsl:text>
                            Show the result set without XSL translation applied.<br/>

                            <xsl:text disable-output-escaping="yes"> 
                                <![CDATA[&nbsp;*&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;]]>
                            </xsl:text> 
                            Show entire contents of table using show_table service. <hr/>'||       

                            --display the rest of the page
                            --using nested for-each loops again, selecting for Tables/u elements
                               ' <xsl:for-each select="Tables/u">
                                  
                                   <h2>User <xsl:value-of select="@user_name"/></h2>'||
                                    '<xsl:for-each select="t">'||
                                       
                                      ' <xsl:text disable-output-escaping="yes"> '||
                                        ' <![CDATA[|&mdash;&nbsp;&nbsp;]]> '||
                                      ' </xsl:text>'||

                                           --create link  1) to qxml demo with the [] square (&oplus;) thingy
                                           ' <a>
                                               <xsl:attribute name="href">
                                                 qxml? SELECT * FROM <xsl:value-of select="@table_name"/>
                                                </xsl:attribute>
                                                <xsl:text disable-output-escaping="yes">
                                                    <![CDATA[&oplus;]]>
                                                </xsl:text>
                                            </a>'||
                                     ' <xsl:text disable-output-escaping="yes"> '||
                                        '<![CDATA[&nbsp;&nbsp;]]>'||
                                      ' </xsl:text>'||

                                           --create link  2) to 20 records with the /\ triangle (&delta;) thingy
                                           ' <a>
                                                <xsl:attribute name="href">
                                                    <![CDATA[show_xml_table?user_name=]]><xsl:value-of select="../@user_name"/><![CDATA[&table_name=]]><xsl:value-of select="@table_name"/><![CDATA[&limit=20&start_at=1]]>'||                                  
                                              ' </xsl:attribute>'||
                                                  ' <xsl:text disable-output-escaping="yes">
                                                        <![CDATA[&Delta;]]>
                                                    </xsl:text>
                                            </a>'||

                                     ' <xsl:text disable-output-escaping="yes"> '||
                                        '<![CDATA[&nbsp;&nbsp;]]>'||
                                      ' </xsl:text>'||

                                           --create link  2) to 20 records with the /\ triangle (&delta;) thingy
                                           ' <a>
                                                <xsl:attribute name="href">
                                                    <![CDATA[show_xml_table?user_name=]]><xsl:value-of select="../@user_name"/><![CDATA[&table_name=]]><xsl:value-of select="@table_name"/><![CDATA[&showxml=TRUE]]>'||                                  
                                              ' </xsl:attribute>'||
                                                  ' <xsl:text disable-output-escaping="yes">
                                                        <![CDATA[&Dagger;]]>
                                                    </xsl:text>
                                            </a>
                                       <xsl:text disable-output-escaping="yes">
                                       <![CDATA[&nbsp;&nbsp;&mdash;]]>
                                      </xsl:text>'||

                                           --create link  3) to all records with the table name
                                           '<a>
                                                <xsl:attribute name="href">
                                                <![CDATA[show_xml_table?user_name=]]><xsl:value-of select="../@user_name"/><![CDATA[&table_name=]]><xsl:value-of select="@table_name"/>
                                                </xsl:attribute>
                                                <xsl:value-of select="@table_name"/><br/>
                                            </a>'||

                                  ' </xsl:for-each>
                                </xsl:for-each>
                            <br/><br/>'||

                         '</xsl:otherwise>'||

                      ' </xsl:choose>'||
                    ' <hr/>'||
        
                    --insert the html code for $yellowtable
                    ' <xsl:copy-of select="$yellowtable"/>
        
                    </body>
                </html>
             </xsl:template>
             </xsl:stylesheet>';

    RETURN;

END;
    
go


--create it 
call sa_make_object( 'service', 'xsldemo' )
go

--alter it with type raw
--(we want to define our own headers etc)
alter service xsldemo type 'raw' authorization on user dba secure off
  as call xsldemo( )
go
